/* 
 *      Copyright 2015 Julien Lavergne <gilir@ubuntu.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */
#if USE_GTK
using Gtk;
using AppIndicator;
using Notify;
#endif

namespace Lxsession 
{
    public class IconObject : GLib.Object
    {
        public string name;
        public string icon_name;    
        public string notification_text;
        public Gtk.Menu menu;

        public Indicator indicator;

        public Notify.Notification notification;
        
        public IconObject(string name_param, string? icon_name_param, string? notification_param, Gtk.Menu? menu_param)
        {
            this.name = name_param;

            if (icon_name_param != null)
            {
                this.icon_name = icon_name_param;
            }
            else
            {
                this.icon_name = "gtk-warning";
            }

            if (notification_param != null)
            {
                this.notification_text = notification_param;
            }

#if USE_GTK
            this.menu = menu_param;

            this.indicator = new Indicator(this.name, this.icon_name,
                              IndicatorCategory.APPLICATION_STATUS);

            this.notification = new Notify.Notification ("LXsession", this.notification_text, this.icon_name);
#endif

        }

        public void init()
        {
#if USE_GTK
            message("Enter notification code");

            if (this.indicator == null)
            {
                this.indicator = new Indicator(this.name, this.icon_name,
                                  IndicatorCategory.APPLICATION_STATUS);
            }

            this.indicator.set_status(IndicatorStatus.ACTIVE);


            if (this.menu != null)
            {
                this.indicator.set_menu(this.menu);
            }


/*
            var launch_menu = new Gtk.MenuItem.with_label(this.launch_menu_text);
            launch_menu.activate.connect(() => {
                try
                {
                    Process.spawn_command_line_async(this.launch_command);
                }
                catch (SpawnError err)
                {
                    warning (err.message);
                }
            });
            launch_menu.show();
            this.menu.append(launch_menu);
*/

            
#endif
//TODO Make it work without GTK
 
        }

        public void activate()
        {
            message("Try activate");
            if (this.indicator != null)
            {
                message("Activate");
                this.indicator.set_status(IndicatorStatus.ACTIVE);
                this.notification.show ();
            }
        }

        public void inactivate()
        {
            message("Try inactivate");
            if (this.indicator != null)
            {
                message("Inactivate");
                this.indicator.set_status(IndicatorStatus.PASSIVE);
            }
        }

        public void set_icon(string param_icon_name)
        {
            this.icon_name = param_icon_name;
            this.indicator.icon_name = param_icon_name;
        }

        public void set_menu(Gtk.Menu param_menu)
        {
            this.menu = param_menu;
            this.indicator.set_menu(param_menu);
        }
        public void set_notification (string notification_param)
        {
            this.notification_text = notification_param;
            this.notification.body = notification_param;
        }
     }
}
